@@ -2,7 +2,6 @@ import re
 
 import django
 import pytest
-from crispy_forms import __version__
 from crispy_forms.bootstrap import (
     AppendedText,
     FieldWithButtons,
@@ -17,18 +16,12 @@ from crispy_forms.utils import render_crispy_form
 from django import forms
 from django.forms.models import formset_factory
 from django.middleware.csrf import _get_new_csrf_string
-from django.template import Context, Template, TemplateSyntaxError
+from django.template import Context, Template
 from django.test import override_settings
 from django.test.html import parse_html
 from django.urls import reverse
 
-from .forms import (
-    SampleForm,
-    SampleForm7,
-    SampleForm8,
-    SampleFormWithMedia,
-    SampleFormWithMultiValueField,
-)
+from .forms import SampleForm, SampleForm7, SampleForm8, SampleFormWithMedia
 from .utils import parse_expected, parse_form
 
 CONVERTERS = {
@@ -38,7 +31,7 @@ CONVERTERS = {
 }
 
 
-def test_inputs(settings):
+def test_inputs():
     form_helper = FormHelper()
     form_helper.add_input(Submit("my-submit", "Submit", css_class="button white"))
     form_helper.add_input(Reset("my-reset", "Reset"))
@@ -62,7 +55,7 @@ def test_inputs(settings):
     assert 'class="btn"' in html
     assert "btn btn-primary" in html
     assert "btn btn-inverse" in html
-    assert len(re.findall(r"<input[^>]+> <", html)) == 8
+    assert len(re.findall(r"<input[^>]+> <", html)) == 9
 
 
 def test_invalid_form_method():
@@ -71,7 +64,7 @@ def test_invalid_form_method():
         form_helper.form_method = "superPost"
 
 
-def test_form_with_helper_without_layout(settings):
+def test_form_with_helper_without_layout():
     form_helper = FormHelper()
     form_helper.form_id = "this-form-rocks"
     form_helper.form_class = "forms-that-rock"
@@ -112,11 +105,8 @@ def test_form_with_helper_without_layout(settings):
     assert 'id="this-form-rocks"' not in html
 
 
-@pytest.mark.skipif(
-    __version__[0] == "1", reason='#1262 fixed required attributes and empty for="".'
-)
 @override_settings(CRISPY_CLASS_CONVERTERS=CONVERTERS)
-def test_form_show_errors_non_field_errors():
+def test_form_show_errors_non_field_errors(settings):
     form = SampleForm({"password1": "wargame", "password2": "god"})
     form.helper = FormHelper()
     form.helper.form_show_errors = True
@@ -132,23 +122,14 @@ def test_form_show_errors_non_field_errors():
     # First we render with errors
     c = Context({"testForm": form})
     # Ensure those errors were rendered
-    if django.VERSION < (4, 1):
-        # Removed "for = ..." from MultiWidget's <label>.
-        # https://github.com/django/django/commit/c6c6cd3c5ad9c36795bb120e521590424f034ae4
+    if django.VERSION < (5, 0):
         expected = parse_expected(
-            "bootstrap3/test_form_helper/"
-            "test_form_show_errors_non_field_errors_true_lte40.html"
-        )
-    elif django.VERSION >= (5, 0):
-        # Added 'aria-describedby' for fields with help_text
-        # https://docs.djangoproject.com/en/5.0/releases/5.0/#forms
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/"
-            "test_form_show_errors_non_field_errors_true_gte50.html"
+            "bootstrap4/test_form_helper/"
+            "test_form_show_errors_non_field_errors_true_lt50.html"
         )
     else:
         expected = parse_expected(
-            "bootstrap3/test_form_helper/"
+            "bootstrap4/test_form_helper/"
             "test_form_show_errors_non_field_errors_true.html"
         )
     assert parse_html(template.render(c)) == expected
@@ -157,57 +138,32 @@ def test_form_show_errors_non_field_errors():
     form.helper.form_show_errors = False
     c = Context({"testForm": form})
     # Ensure errors were not rendered
-    if django.VERSION < (4, 1):
+    if django.VERSION < (5, 0):
         expected = parse_expected(
-            "bootstrap3/test_form_helper/"
-            "test_form_show_errors_non_field_errors_false_lte40.html"
-        )
-    elif django.VERSION >= (5, 0):
-        # Added 'aria-describedby' for fields with help_text
-        # https://docs.djangoproject.com/en/5.0/releases/5.0/#forms
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/"
-            "test_form_show_errors_non_field_errors_false_gte50.html"
+            "bootstrap4/test_form_helper/"
+            "test_form_show_errors_non_field_errors_false_lt50.html"
         )
     else:
         expected = parse_expected(
-            "bootstrap3/test_form_helper/"
+            "bootstrap4/test_form_helper/"
             "test_form_show_errors_non_field_errors_false.html"
         )
     assert parse_html(template.render(c)) == expected
 
 
-def test_html5_required():
-    form = SampleForm()
-    form.helper = FormHelper()
-    form.helper.html5_required = True
-    html = render_crispy_form(form)
-    # 6 out of 7 fields are required and an extra one for
-    # the SplitDateTimeWidget makes 7.
-    if django.VERSION < (1, 10):
-        assert html.count('required="required"') == 7
-    else:
-        assert len(re.findall(r"\brequired\b", html)) == 7
-
-    form = SampleForm()
-    form.helper = FormHelper()
-    form.helper.html5_required = False
-    html = render_crispy_form(form)
-
-
-def test_media_is_included_by_default_with_bootstrap3():
+def test_media_is_included_by_default_with_bootstrap4():
     form = SampleFormWithMedia()
     form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap3"
+    form.helper.template_pack = "bootstrap4"
     html = render_crispy_form(form)
     assert "test.css" in html
     assert "test.js" in html
 
 
-def test_media_removed_when_include_media_is_false_with_bootstrap3():
+def test_media_removed_when_include_media_is_false_with_bootstrap4():
     form = SampleFormWithMedia()
     form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap3"
+    form.helper.template_pack = "bootstrap4"
     form.helper.include_media = False
     html = render_crispy_form(form)
     assert "test.css" not in html
@@ -267,7 +223,7 @@ def test_template_helper_access():
     assert helper["form_id"] == "test-form"
 
 
-def test_without_helper(settings):
+def test_without_helper():
     template = Template(
         """
         {% load crispy_forms_tags %}
@@ -283,31 +239,7 @@ def test_without_helper(settings):
     assert "action" not in html
 
 
-def test_template_pack_override_wrong():
-    with pytest.raises(TemplateSyntaxError):
-        Template(
-            """
-            {% load crispy_forms_tags %}
-            {% crispy form 'foo' %}
-        """
-        )
-
-
-def test_invalid_helper(settings):
-    template = Template(
-        """
-        {% load crispy_forms_tags %}
-        {% crispy form form_helper %}
-    """
-    )
-    c = Context({"form": SampleForm(), "form_helper": "invalid"})
-
-    settings.CRISPY_FAIL_SILENTLY = settings.TEMPLATE_DEBUG = False
-    with pytest.raises(TypeError):
-        template.render(c)
-
-
-def test_formset_with_helper_without_layout(settings):
+def test_formset_with_helper_without_layout():
     template = Template(
         """
         {% load crispy_forms_tags %}
@@ -358,8 +290,8 @@ def test_CSRF_token_POST_form():
 
     # The middleware only initializes the CSRF token when processing a real request
     # So using RequestContext or csrf(request) here does not work.
-    # Instead I set the key `csrf_token` to a CSRF token manually,
-    # which `csrf_token` tag uses
+    # Instead I set the key `csrf_token` to a CSRF token manually, which `csrf_token`
+    # tag uses
     c = Context(
         {
             "form": SampleForm(),
@@ -521,7 +453,7 @@ def test_helper_std_field_template_no_layout():
 
 
 @override_settings(CRISPY_CLASS_CONVERTERS=CONVERTERS)
-def test_bootstrap_form_show_errors_bs3():
+def test_bootstrap_form_show_errors_bs4():
     form = SampleForm(
         {
             "email": "invalidemail",
@@ -540,33 +472,26 @@ def test_bootstrap_form_show_errors_bs3():
         PrependedText("password2", "blabla"),
     )
     form.is_valid()
-
     form.helper.form_show_errors = True
-    if django.VERSION >= (5, 0):
-        # Added 'aria-describedby' for fields with help_text
-        # https://docs.djangoproject.com/en/5.0/releases/5.0/#forms
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/bootstrap_form_show_errors_bs3_true_gte50.html"
+    if django.VERSION < (5, 0):
+        expected = (
+            "bootstrap4/test_form_helper/bootstrap_form_show_errors_bs4_true_lt50.html"
         )
     else:
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/bootstrap_form_show_errors_bs3_true.html"
+        expected = (
+            "bootstrap4/test_form_helper/bootstrap_form_show_errors_bs4_true.html"
         )
-    assert parse_form(form) == expected
-
+    assert parse_form(form) == parse_expected(expected)
     form.helper.form_show_errors = False
-    if django.VERSION >= (5, 0):
-        # Added 'aria-describedby' for fields with help_text
-        # https://docs.djangoproject.com/en/5.0/releases/5.0/#forms
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/"
-            "bootstrap_form_show_errors_bs3_false_gte50.html"
+    if django.VERSION < (5, 0):
+        expected = (
+            "bootstrap4/test_form_helper/bootstrap_form_show_errors_bs4_false_lt50.html"
         )
     else:
-        expected = parse_expected(
-            "bootstrap3/test_form_helper/bootstrap_form_show_errors_bs3_false.html"
+        expected = (
+            "bootstrap4/test_form_helper/bootstrap_form_show_errors_bs4_false.html"
         )
-    assert parse_form(form) == expected
+    assert parse_form(form) == parse_expected(expected)
 
 
 def test_error_text_inline(settings):
@@ -581,9 +506,8 @@ def test_error_text_inline(settings):
     form.is_valid()
     html = render_crispy_form(form)
 
-    help_class = "help-inline"
-    help_tag_name = "p"
-    help_class = "help-block"
+    help_class = "invalid-feedback"
+    help_tag_name = "div"
 
     matches = re.findall(
         r'<span id="error_\d_\w*" class="%s"' % help_class, html, re.MULTILINE
@@ -596,7 +520,8 @@ def test_error_text_inline(settings):
     form.helper.error_text_inline = False
     html = render_crispy_form(form)
 
-    help_class = "help-block"
+    help_class = "invalid-feedback"
+    help_tag_name = "p"
 
     matches = re.findall(
         r'<{} id="error_\d_\w*" class="{}"'.format(help_tag_name, help_class),
@@ -606,7 +531,7 @@ def test_error_text_inline(settings):
     assert len(matches) == 3
 
 
-def test_error_and_help_inline_bootstrap3():
+def test_error_and_help_inline():
     form = SampleForm({"email": "invalidemail"})
     form.helper = FormHelper()
     form.helper.error_text_inline = False
@@ -617,7 +542,7 @@ def test_error_and_help_inline_bootstrap3():
 
     # Check that help goes before error, otherwise CSS won't work
     help_position = html.find('<span id="hint_id_email" class="help-inline">')
-    error_position = html.find('<p id="error_1_id_email" class="help-block">')
+    error_position = html.find('<p id="error_1_id_email" class="invalid-feedback">')
     assert help_position < error_position
 
     # Viceversa
@@ -631,7 +556,7 @@ def test_error_and_help_inline_bootstrap3():
 
     # Check that error goes before help, otherwise CSS won't work
     error_position = html.find('<span id="error_1_id_email" class="help-inline">')
-    help_position = html.find('<div id="hint_id_email" class="help-block">')
+    help_position = html.find('<small id="hint_id_email" class="form-text text-muted">')
     assert error_position < help_position
 
 
@@ -651,36 +576,71 @@ def test_form_show_labels():
     assert html.count("<label") == 0
 
 
-def test_label_class_and_field_class():
+def test_label_class_and_field_class_bs4():
     form = SampleForm()
     form.helper = FormHelper()
     form.helper.label_class = "col-lg-2"
     form.helper.field_class = "col-lg-8"
     html = render_crispy_form(form)
-    dom = parse_html(html)
 
-    snippet = parse_html(
-        '<div class="form-group"> <div class="controls col-lg-offset-2 col-lg-8"> '
-        '<div id="div_id_is_company" class="checkbox"> '
-        '<label for="id_is_company" class=""><input class="checkboxinput" '
-        'id="id_is_company" name="is_company" type="checkbox" />company'
-    )
-    assert dom.count(snippet)
+    assert '<div class="form-group">' in html
+    assert '<div class="col-lg-8">' in html
     assert html.count("col-lg-8") == 7
+    assert "offset" not in html
 
     form.helper.label_class = "col-sm-3 col-md-4"
     form.helper.field_class = "col-sm-8 col-md-6"
     html = render_crispy_form(form)
-    dom = parse_html(html)
 
-    snippet = parse_html(
-        '<div class="form-group"> <div class="controls col-sm-offset-3 col-md-offset-4 '
-        'col-sm-8 col-md-6"><div id="div_id_is_company" class="checkbox"> '
-        '<label for="id_is_company" class=""><input class="checkboxinput" '
-        'id="id_is_company" name="is_company" type="checkbox" />company'
-    )
-    assert dom.count(snippet)
+    assert '<div class="form-group">' in html
+    assert '<div class="col-sm-8 col-md-6">' in html
     assert html.count("col-sm-8") == 7
+    assert "offset" not in html
+
+
+def test_label_class_and_field_class_bs4_offset_when_horizontal():
+    # Test col-XX-YY pattern
+    form = SampleForm()
+    form.helper = FormHelper()
+    form.helper.label_class = "col-lg-2"
+    form.helper.field_class = "col-lg-8"
+    form.helper.form_class = "form-horizontal"
+    html = render_crispy_form(form)
+
+    assert '<div class="form-group row">' in html
+    assert '<div class="offset-lg-2 col-lg-8">' in html
+    assert html.count("col-lg-8") == 7
+
+    # Test multi col-XX-YY pattern and col-X pattern
+
+    form.helper.label_class = "col-sm-3 col-md-4 col-5 col-lg-4"
+    form.helper.field_class = "col-sm-8 col-md-6 col-7 col-lg-8"
+    html = render_crispy_form(form)
+
+    assert '<div class="form-group row">' in html
+    assert (
+        '<div class="offset-sm-3 offset-md-4 offset-5 offset-lg-4 col-sm-8 '
+        'col-md-6 col-7 col-lg-8">' in html
+    )
+    assert html.count("col-sm-8") == 7
+    assert html.count("col-md-6") == 7
+    assert html.count("col-7") == 7
+    assert html.count("col-lg-8") == 7
+
+
+def test_form_group_with_form_inline_bs4():
+    form = SampleForm()
+    form.helper = FormHelper()
+    html = render_crispy_form(form)
+    assert '<div class="form-group">' in html
+
+    # .row class shouldn't be together with .form-group in inline forms
+    form = SampleForm()
+    form.helper = FormHelper()
+    form.helper.form_class = "form-inline"
+    form.helper.field_template = "bootstrap4/layout/inline_field.html"
+    html = render_crispy_form(form)
+    assert '<div class="form-group row">' not in html
 
 
 def test_passthrough_context():
@@ -697,19 +657,3 @@ def test_passthrough_context():
     html = render_crispy_form(form, helper=form.helper, context=c)
     assert "Got prefix: foo" in html
     assert "Got suffix: bar" in html
-
-
-def test_bootstrap3_does_add_form_control_class_to_non_multivaluefield():
-    form = SampleForm()
-    form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap3"
-    html = render_crispy_form(form)
-    assert "form-control" in html
-
-
-def test_bootstrap3_does_not_add_form_control_class_to_multivaluefield():
-    form = SampleFormWithMultiValueField()
-    form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap3"
-    html = render_crispy_form(form)
-    assert "form-control" not in html
